@@ -1,4 +1,3 @@
-from .utils import appdir
 import g4f
 from importlib import metadata
 import logging
@@ -13,21 +12,15 @@ __repo__ = "https://github.com/Simatwa/python-tgpt"
 
 tgpt_providers = [
     "auto",
-    "leo",
     "openai",
-    "opengpt",
     "koboldai",
-    "gemini",
     "phind",
-    "llama2",
     "blackboxai",
     "gpt4all",
-    "webchatgpt",
     "g4fauto",
     "poe",
     "groq",
     "perplexity",
-    "yepchat",
     "novita",
 ]
 
