@@ -14,6 +14,9 @@ import time
 import subprocess
 from threading import Thread as thr
 from functools import wraps
+
+# rich
+
 from rich.panel import Panel
 from rich.style import Style
 from rich.markdown import Markdown
@@ -22,7 +25,11 @@ from rich.live import Live
 from rich.table import Table
 from rich.prompt import Prompt
 from rich.progress import Progress
+
 from typing import Iterable
+
+# pytgpt
+
 from pytgpt.utils import Optimizers
 from pytgpt.utils import default_path
 from pytgpt.utils import AwesomePrompts
@@ -30,11 +37,19 @@ from pytgpt.utils import RawDog
 from pytgpt.imager import Imager
 from pytgpt.imager import Prodia
 from pytgpt.utils import Audio
-from WebChatGPT.console import chat as webchatgpt
+from pytgpt.utils import suggest_query
+
+# colorama
 from colorama import Fore
 from colorama import init as init_colorama
+
 from dotenv import load_dotenv
 
+# prompt-toolkit
+from prompt_toolkit import PromptSession
+from prompt_toolkit.completion import ThreadedCompleter, Completer, Completion
+from prompt_toolkit.document import Document
+
 init_colorama(autoreset=True)
 
 load_dotenv()  # loads .env variables
@@ -330,6 +345,42 @@ class busy_bar:
         return decorator
 
 
+class CustomCompleter(Completer):
+    """Suggests query based on user prompts"""
+
+    def __init__(
+        self,
+        caller: object,
+        suggestions_limit: int = 15,
+        special_commands: list[str] = [],
+    ):
+        self.suggestions_limit = suggestions_limit
+        self.caller = caller
+        self.special_commands = special_commands
+
+    def get_completions(self, document: Document, complete_event):
+        word = document.text
+        if word and self.suggestions_limit > 0 and not word.startswith("./"):
+            completions = []
+            first_word = word.strip().split(" ")[0]
+            if first_word in self.special_commands:
+                completions.append(
+                    Completion(
+                        f"{first_word} [RESERVED] "
+                        + getattr(self.caller, f"do_{first_word}").__doc__
+                    )
+                )
+                return completions
+            for count, suggestion in enumerate(
+                suggest_query(word, timeout=2, die_silently=True), start=1
+            ):
+                completions.append(Completion(suggestion, start_position=-len(word)))
+                if count >= self.suggestions_limit:
+                    break
+            return completions
+        return []
+
+
 class Main(cmd.Cmd):
     intro = (
         "Welcome to AI Chat in terminal. "
@@ -361,6 +412,8 @@ class Main(cmd.Cmd):
         internal_exec=False,
         confirm_script=False,
         interpreter="python",
+        suggestions_limit=15,
+        non_interactive=False,
         *args,
         **kwargs,
     ):
@@ -431,26 +484,6 @@ class Main(cmd.Cmd):
                         "Consider running 'pytgpt gpt4free test -y' first"
                     )
 
-            elif provider == "leo":
-                import pytgpt.leo as leo
-
-                self.bot = leo.LEO(
-                    is_conversation=disable_conversation,
-                    max_tokens=max_tokens,
-                    temperature=temperature,
-                    top_k=top_k,
-                    top_p=top_p,
-                    model=getOr(model, leo.main.model),
-                    brave_key=getOr(auth, leo.main.key),
-                    timeout=timeout,
-                    intro=intro,
-                    filepath=filepath,
-                    update_file=update_file,
-                    proxies=proxies,
-                    history_offset=history_offset,
-                    act=awesome_prompt,
-                )
-
             elif provider == "openai":
                 assert auth, (
                     "OpenAI's API-key is required. " "Use the flag `--key` or `-k`"
@@ -475,21 +508,6 @@ class Main(cmd.Cmd):
                     act=awesome_prompt,
                 )
 
-            elif provider == "opengpt":
-                from pytgpt.opengpt import OPENGPT
-
-                self.bot = OPENGPT(
-                    is_conversation=disable_conversation,
-                    max_tokens=max_tokens,
-                    timeout=timeout,
-                    intro=intro,
-                    filepath=filepath,
-                    update_file=update_file,
-                    proxies=proxies,
-                    history_offset=history_offset,
-                    act=awesome_prompt,
-                )
-
             elif provider == "koboldai":
                 from pytgpt.koboldai import KOBOLDAI
 
@@ -507,19 +525,6 @@ class Main(cmd.Cmd):
                     act=awesome_prompt,
                 )
 
-            elif provider == "gemini":
-                from pytgpt.gemini import GEMINI
-
-                assert auth, (
-                    "Path to gemini.google.com.cookies.json file is required. "
-                    "Use the flag `--key` or `-k`"
-                )
-                self.bot = GEMINI(
-                    cookie_file=auth,
-                    proxy=proxies,
-                    timeout=timeout,
-                )
-
             elif provider == "phind":
                 import pytgpt.phind.main as phind
 
@@ -537,26 +542,6 @@ class Main(cmd.Cmd):
                     quiet=quiet,
                 )
 
-            elif provider == "llama2":
-                import pytgpt.llama2.main as llama2
-
-                self.bot = llama2.LLAMA2(
-                    is_conversation=disable_conversation,
-                    max_tokens=max_tokens,
-                    temperature=temperature,
-                    presence_penalty=top_p,
-                    frequency_penalty=top_k,
-                    top_p=top_p,
-                    model=getOr(model, llama2.default_model),
-                    timeout=timeout,
-                    intro=intro,
-                    filepath=filepath,
-                    update_file=update_file,
-                    proxies=proxies,
-                    history_offset=history_offset,
-                    act=awesome_prompt,
-                )
-
             elif provider == "blackboxai":
 
                 from pytgpt.blackboxai import BLACKBOXAI
@@ -573,26 +558,6 @@ class Main(cmd.Cmd):
                     act=awesome_prompt,
                 )
 
-            elif provider == "yepchat":
-                from pytgpt.yepchat import main as yepchat
-
-                self.bot = yepchat.YEPCHAT(
-                    is_conversation=disable_conversation,
-                    max_tokens=max_tokens,
-                    temperature=temperature,
-                    presence_penalty=top_p,
-                    frequency_penalty=top_k,
-                    top_p=top_p,
-                    model=getOr(model, yepchat.model),
-                    timeout=timeout,
-                    intro=intro,
-                    filepath=filepath,
-                    update_file=update_file,
-                    proxies=proxies,
-                    history_offset=history_offset,
-                    act=awesome_prompt,
-                )
-
             elif provider == "gpt4all":
                 assert auth, (
                     "Path to LLM (.gguf or .bin) file is required. "
@@ -617,35 +582,6 @@ class Main(cmd.Cmd):
                     act=awesome_prompt,
                 )
 
-            elif provider == "webchatgpt":
-                assert auth, (
-                    "Path to `chat.openai.cookies.com.json` is required. "
-                    "Use the flag `--key` or `-k`"
-                )
-
-                from pytgpt.webchatgpt import main
-
-                self.bot = main.WEBCHATGPT(
-                    cookie_file=auth,
-                    model=getOr(model, main.default_model),
-                    proxy=proxies,
-                    timeout=timeout,
-                    filepath=filepath,
-                    update_file=update_file,
-                    intro=intro,
-                    act=awesome_prompt,
-                )
-                intro_response = self.bot.chat(self.bot.conversation.intro, stream=True)
-                if not quiet:
-                    this.stream_output(
-                        intro_response,
-                        title="Intro Response",
-                        is_markdown=True,
-                        style=Style(
-                            color="cyan",
-                        ),
-                    )
-
             elif provider == "poe":
                 assert auth, (
                     "Path to poe.com.cookies.json file or 'p-b' cookie-value is required. "
@@ -772,6 +708,37 @@ class Main(cmd.Cmd):
         self.read_aloud = False
         self.read_aloud_voice = "Brian"
         self.path_to_last_response_audio = None
+        if not non_interactive:
+            self.completer_session = PromptSession(
+                "",
+                completer=ThreadedCompleter(
+                    CustomCompleter(
+                        self,
+                        suggestions_limit,
+                        [
+                            "cd",
+                            "copy_this",
+                            "h",
+                            "last_response",
+                            "rawdog",
+                            "settings",
+                            "with_copied",
+                            "clear",
+                            "exec",
+                            "help",
+                            "load",
+                            "reread",
+                            "shell",
+                            "code",
+                            "exit",
+                            "history",
+                            "new_intro",
+                            "reset",
+                            "sys",
+                        ],
+                    )
+                ),
+            )
         self.__init_time = time.time()
         self.__start_time = time.time()
         self.__end_time = time.time()
@@ -802,7 +769,7 @@ class Main(cmd.Cmd):
                 f"~[`{Fore.LIGHTWHITE_EX}🕒{Fore.BLUE}{current_time}-`"
                 f"{Fore.LIGHTWHITE_EX}💻{Fore.RED}{find_range(self.__init_time, time.time(), True)}-`"
                 f"{Fore.LIGHTWHITE_EX}⚡{Fore.YELLOW}{find_range(self.__start_time, self.__end_time)}s]`"
-                f"\n╰─>"
+                # f"\n╰─>"
             )
             whitelist = ["[", "]", "~", "-", "(", ")"]
             for character in whitelist:
@@ -815,9 +782,71 @@ class Main(cmd.Cmd):
                 f"~[🕒{current_time}"
                 f"-💻{find_range(self.__init_time, time.time(), True)}"
                 f"-⚡{find_range(self.__start_time, self.__end_time)}s]"
-                "\n╰─>"
+                # "\n╰─>"
             )
 
+    def cmdloop(self, intro=None):
+        """Repeatedly issue a prompt, accept input, parse an initial prefix
+        off the received input, and dispatch to action methods, passing them
+        the remainder of the line as argument.
+
+        """
+
+        self.preloop()
+        if self.use_rawinput and self.completekey:
+            try:
+                import readline
+
+                self.old_completer = readline.get_completer()
+                readline.set_completer(self.complete)
+                if hasattr(readline, "backend") and readline.backend == "editline":
+                    if self.completekey == "tab":
+                        # libedit uses "^I" instead of "tab"
+                        command_string = "bind ^I rl_complete"
+                    else:
+                        command_string = f"bind {self.completekey} rl_complete"
+                else:
+                    command_string = f"{self.completekey}: complete"
+                readline.parse_and_bind(command_string)
+            except ImportError:
+                pass
+        try:
+            if intro is not None:
+                self.intro = intro
+            if self.intro:
+                self.stdout.write(str(self.intro) + "\n")
+            stop = None
+            while not stop:
+                if self.cmdqueue:
+                    line = self.cmdqueue.pop(0)
+                else:
+                    if self.use_rawinput:
+                        try:
+                            print(self.prompt, end="")
+                            line = self.completer_session.prompt("\n╰─>")
+                        except EOFError:
+                            line = "EOF"
+                    else:
+                        self.stdout.write(self.prompt)
+                        self.stdout.flush()
+                        line = self.stdin.readline()
+                        if not len(line):
+                            line = "EOF"
+                        else:
+                            line = line.rstrip("\r\n")
+                line = self.precmd(line)
+                stop = self.onecmd(line)
+                stop = self.postcmd(stop, line)
+            self.postloop()
+        finally:
+            if self.use_rawinput and self.completekey:
+                try:
+                    import readline
+
+                    readline.set_completer(self.old_completer)
+                except ImportError:
+                    pass
+
     def output_bond(
         self,
         title: str,
@@ -1244,7 +1273,7 @@ class Main(cmd.Cmd):
     def do_exit(self, line):
         """Quit this program"""
         if click.confirm("Are you sure to exit"):
-            click.secho("Okay Goodbye!", fg="yellow")
+            click.secho("^-^ Okay Goodbye!", fg="yellow")
             return True
 
 
@@ -1422,6 +1451,13 @@ class ChatInteractive:
             "for advanced g4f providers test"
         ),
     )
+    @click.option(
+        "-sl",
+        "--suggestions-limit",
+        type=click.INT,
+        help="Prompt suggestions limit - 0 to disable suggestion",
+        default=15,
+    )
     @click.option(
         "-vo",
         "--vertical-overflow",
@@ -1530,6 +1566,7 @@ class ChatInteractive:
         awesome_prompt,
         proxy_path,
         provider,
+        suggestions_limit,
         vertical_overflow,
         whole,
         quiet,
@@ -1570,6 +1607,7 @@ class ChatInteractive:
             internal_exec=internal_exec,
             confirm_script=confirm_script,
             interpreter=interpreter,
+            suggestions_limit=suggestions_limit,
         )
         busy_bar.spin_index = busy_bar_index
         bot.code_theme = code_theme
@@ -1869,6 +1907,7 @@ class ChatGenerate:
             internal_exec=internal_exec,
             confirm_script=confirm_script,
             interpreter=interpreter,
+            non_interactive=True,
         )
         prompt = prompt if prompt else ""
         copied_placeholder = "{{copied}}"
@@ -2613,10 +2652,6 @@ class API:
 
 def make_commands():
     """Make pytgpt chained commands"""
-    # webchatgpt
-    # Intergration with WebChatGPT https://github.com/Simatwa/WebChatGPT
-    EntryGroup.tgpt2_.add_command(webchatgpt, "webchatgpt")
-
     # generate
     EntryGroup.tgpt2_.add_command(ChatGenerate.generate)
 
